## -*- coding: utf-8 -*-
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# Python import
from itertools import imap
import logging

# Zope imports
from AccessControl import ClassSecurityInfo
from zope.interface import Interface, implements

# CMF Imports
from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import getToolByName

# Products imports
from Products.ATContentTypes.content.folder import ATFolder
from Products.FaceBook.config import PROJECTNAME
from Products.FaceBook import config

# Archetypes imports
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

LOG = logging.getLogger("FaceBook.folder")

class IFaceFolder(Interface):
    pass

class Folder(ATFolder):
    """
    Folder containing personal cards and other face folders
    """
    implements(IFaceFolder)

    portal_type = meta_type = 'FaceFolder'
    archetype_name = 'face folder'
    typeDescription = 'Folder containing personal cards and other face folders.'
    default_view = 'facefolder'

    filter_content_types = True
    allowed_content_types = ('FaceFolder', 'PersonalCard')
    security = ClassSecurityInfo()

    folder_content_type = 'FaceFolder'
    card_content_type   = 'PersonalCard'

    security.declareProtected(CCP.View, "getPersonalCards")
    def getPersonalCards(self):
        """
        Returns dictionary of the personal cards.
        It returns the cards of this folder and folders directly under this
        one, grouped by folder :
        [(folder_title, [personalcard1, ...]), ...]
        """
        face_catalog = getToolByName(self, 'face_catalog')
        portal_catalog = getToolByName(self, 'portal_catalog')
        pathquery = {'query': '/'.join(self.getPhysicalPath()),
                     'depth': 1}
        root_cards = face_catalog.searchResults(**{'sort_on': 'sortable_name',
                                'path': pathquery})
        sub_folders = portal_catalog.searchResults(**{'sort_on': 'getObjPositionInParent',
                         'portal_type': self.folder_content_type,
                         'path': pathquery,
                         })
        if root_cards:
            result = [('__root__', self.absolute_url(), root_cards)]
        else:
            result = []
        for sub_folder in sub_folders:
            query = {'sort_on': 'sortable_name',
                     'path': {'query': sub_folder.getPath(),
                              'depth': 1}}
            cards = face_catalog.searchResults(**query)
            if cards and len(cards) > 0:
                result.append((sub_folder.Title, sub_folder.getURL(), cards))

        return result

    security.declareProtected(CCP.View, "searchByName")
    def searchByName(self, name):
        """
        Returns a list of personal cards, which name of surname is identical
        to the name parameter.
        """
        catalog = getToolByName(self, config.CATALOG_NAME)

        if not name:
            return []

        query = {'SearchableName': name,
                 'sort_on': 'sortable_name',
                 'portal_type': self.card_content_type,
                 'path': '/'.join(self.getPhysicalPath())}

        brains = catalog.searchResults(**query)
        return brains

    security.declarePrivate("_getRoot")
    def _getRoot(self):
        """
        Return the root of the facefolders (which also is a facefolder)
        """
        parent = self.aq_inner.aq_parent
        if IFaceFolder.providedBy(parent):
            return parent._getRoot()
        return self

    security.declareProtected(CCP.View, "getNavTree")
    def getNavTree(self):
        """
        Return a dictionary containing the navtree.
        The navtree starts at the root of the face folders and display
        two levels of face folders.
        The navtree looks like [(folder_brain, [folder_brain,...]), ...]
        """
        UID = self.UID()
        parent_UID = self.aq_inner.aq_parent.UID()
        root = self._getRoot()

        contentFilter = {'sort_on': 'getObjPositionInParent',
                         'portal_type': self.folder_content_type}

        folder_brains = root.getFolderContents(contentFilter)

        results = []

        def folderWrapping(brain):
            return { 'title' : brain.Title or brain.getId,
                     'url'   : brain.getURL(),
                    }

        for brain in folder_brains:
            folder = folderWrapping(brain)
            folder_object = brain.getObject()
            folder_UID = folder_object.UID()
            subfolder_brains = folder_object.getFolderContents(contentFilter)
            subfolders = imap(folderWrapping, subfolder_brains)
            results.append( {'folder'     : folder,
                             'open'       : folder_UID in (UID, parent_UID),
                             'subfolders' : subfolders } )
        return results

registerType(Folder, PROJECTNAME)
